/*
 * 作成日: 2006/12/08
 *
 * この生成されたコメントの挿入されるテンプレートを変更するため
 * ウィンドウ > 設定 > Java > コード生成 > コードとコメント
 */
package jp.maxwell.applet.simson;

import java.awt.*;
import java.awt.geom.*;

/**
 * @author $Id:$
 */
public class SimsonCanvas extends Canvas {
	private double diameterRatio = 0.6;
	// 軸・単位円の色
	private Color basePartsColor = Color.DARK_GRAY;
	// 三角形の色
	private Color triangleColor = Color.RED;
	// シムソン線の色
	private Color simsonLineColor = Color.BLUE;
	// 九点円の色
	private Color ninePointCircleColor = Color.GREEN;
	
	/**
	 * 基本部品を描画する。
	 */
	public void drawBase(){
		int x0 = this.getWidth() / 2;
		int y0 = this.getHeight() / 2;
		int r =  (int)this.getRadius();
		
		Graphics g = this.getGraphics();
		
		// 軸を描く
		g.setColor( this.basePartsColor);
		g.drawLine( 0, y0, this.getWidth(), y0);
		g.drawLine( x0, 0, x0, this.getHeight());
		// 単位円を描く
		g.drawOval( x0 - r, y0 - r, 2*r, 2*r);
	}

	/**
	 * SimsonLine を描くための三角形を描きます。
	 * @param s
	 */
	public void drawTriangle( SimsonLine s){
		Graphics g = this.getGraphics();
		
		// 半径
		double r = this.getRadius();

		// 原点のスクリーン座標
		Point o = this.getOrigin();
		
		// 三頂点の座標を得る
		Point2D.Double pa = s.getA();
		Point2D.Double pb = s.getB();
		Point2D.Double pc = s.getC();
		
		// スクリーン座標を得る
		Point sa = new Point( o.x + (int)(r*pa.x), o.y - (int)(r*pa.y));
		Point sb = new Point( o.x + (int)(r*pb.x), o.y - (int)(r*pb.y));
		Point sc = new Point( o.x + (int)(r*pc.x), o.y - (int)(r*pc.y));
		
		// 辺を描画する
		g.setColor( this.triangleColor);
		g.drawLine( sa.x, sa.y, sb.x, sb.y);
		g.drawLine( sb.x, sb.y, sc.x, sc.y);
		g.drawLine( sc.x, sc.y, sa.x, sa.y);
	}

	/**
	 * 九点円を描きます。
	 * @param s
	 */
	public void drawNinePointCircle( SimsonLine s){
		Graphics g = this.getGraphics();
		
		// 原点のスクリーン座標
		Point o = this.getOrigin();
		
		// 単位円の半径
		double r = this.getRadius();
		
		// 九点円の中心座標
		Point2D.Double npc = NinePointCircle.getNinePointCircleCenter( s.getAlpha(), s.getBeta(), s.getGamma());
		
		// スクリーン座標を得る
		Point sn = new Point( o.x + (int)(r*npc.x), o.y - (int)(r*npc.y));

		// 九点円を描画
		g.setColor( this.ninePointCircleColor);
		g.drawOval( sn.x-(int)(r*0.5), sn.y-(int)(r*0.5), (int)r, (int)r);
	}

	/**
	 * SimsonLine を描きます。
	 * @param s
	 */
	public void drawSimsonLine( SimsonLine s){
		Graphics g = this.getGraphics();
		
		// 原点のスクリーン座標
		Point o = this.getOrigin();

		// 単位円の半径
		double r = this.getRadius();
		
		// 拡大率
		double m = 1/this.getDiameterRatio();
		
		// 点 D のスクリーン座標を得る。
		Point pd = new Point( o.x + (int)(r*s.getD().x), o.y - (int)(r*s.getD().y));

		// 点 D を描画
		g.setColor( Color.MAGENTA);
		g.fillOval( pd.x - 2, pd.y - 2, 4, 4);
		
		Point s1 = new Point( (int)(-m*r) + o.x, -(int)(r*s.getPointOnSimsonLineFromX( -m)) + o.y);  				
		Point s2 = new Point( (int)(m*r) + o.x, -(int)(r*s.getPointOnSimsonLineFromX( m)) + o.y);
		 
		// シムソン線を描画
		g.setColor( this.simsonLineColor);
		g.drawLine( s1.x, s1.y, s2.x, s2.y);
	}

	/**
	 * スクリーン原点の座標を得ます。
	 * @return
	 */
	public Point getOrigin(){
		return ( new Point( this.getWidth()/2, this.getHeight()/2));
	}
	
	/*
	 * 描画します。 
	 */
	public void paint( Graphics g){
		this.drawBase();
	}
	
	/**
	 * 画面をクリアします。
	 *
	 */
	public void clear(){
		Graphics g = this.getGraphics();
		
		g.setColor( this.getBackground());
		g.fillRect( 0, 0, this.getWidth(), this.getHeight());
	}
	
	/**
	 * 単位円の半径を得ます。
	 * @return
	 */
	private double getRadius(){
		double r = 0;
		int w = this.getWidth();
		int h = this.getHeight();

		if( w > h){
			r = h/2.0;
		}
		else{
			r = w/2.0;
		}
		
		r = (this.getDiameterRatio() * r);
		
		return r;
	}
	
	/**
	 * 画面幅と単位円の直径の比を返します。
	 * @return
	 */
	public double getDiameterRatio(){
		return this.diameterRatio;
	}
	
	/**
	 * 画面幅と単位円の直径の比を設定します。
	 *
	 */
	public void setDiameterRatio( double a){
		this.diameterRatio = a;
	}
}
