/*
 * 作成日: 2006/12/08
 *
 * この生成されたコメントの挿入されるテンプレートを変更するため
 * ウィンドウ > 設定 > Java > コード生成 > コードとコメント
 */
package jp.maxwell.applet.simson;

import java.awt.geom.*;

/**
 * @author $Id:$
 */
public class SimsonLine {
	private double alpha;
	private double beta;
	private double gamma;
	private double t;
	
	public SimsonLine( double alpha, double beta, double gamma, double t){
		this.alpha = alpha;
		this.beta = beta;
		this.gamma = gamma;
		this.t = t;
	}

	/**
	 * 角度 &alpha; を返します。
	 * @return
	 */
	public double getAlpha(){
		return this.alpha;
	}

	/**
	 * 角度 &beta; を返します。
	 * @return
	 */
	public double getBeta(){
		return this.beta;
	}

	/**
	 * 角度 &gamma; を返します。
	 * @return
	 */
	public double getGamma(){
		return this.gamma;
	}
	
	/**
	 * 角度 t を返します。
	 * @return
	 */
	public double getT(){
		return this.t;
	}

	/**
	 * 角度 t を設定します。
	 * @param t
	 */
	public void setT( double t){
		this.t = t;
	}

	/**
	 * 点 A の座標を得ます。
	 */
	public Point2D.Double getA(){
		return new Point2D.Double( Math.cos( this.alpha), Math.sin( this.alpha));
	}

	/**
	 * 点 B の座標を得ます。
	 * @return
	 */
	public Point2D.Double getB(){
		return new Point2D.Double( Math.cos( this.beta), Math.sin( this.beta));
	}

	/**
	 * 点 C の座標を得ます。
	 * @return
	 */
	public Point2D.Double getC(){
		return new Point2D.Double( Math.cos( this.gamma), Math.sin( this.gamma));
	}

	/**
	 * 点 D の座標を得ます。
	 * @return
	 */
	public Point2D.Double getD(){
		return new Point2D.Double( Math.cos( this.t), Math.sin( this.t));
	}

	/**
	 * 点 R の座標を得ます。
	 * @return
	 */
	public Point2D.Double getR(){
		double L = this.getL();
		double x = ((L*(Math.sin( this.t) - Math.sin( this.alpha)) + L*L*Math.cos( this.alpha) + Math.cos( this.t)))/(L*L+1);
		double y = ((L*(Math.cos( this.t) - Math.cos( this.alpha)) + L*L*Math.sin( this.t) + Math.sin( this.alpha)))/(L*L+1);
		
		Point2D.Double p = new Point2D.Double( x, y);
		
		return p;
	}

	/**
	 * 点 Q の座標を得ます。
	 * @return
	 */
	public Point2D.Double getQ(){
		double M = this.getM();
		double x = ((M*(Math.sin( this.t) - Math.sin(this.alpha)) + M*M*Math.cos(this.alpha) + Math.cos( this.t)))/(M*M+1);
		double y = ((M*(Math.cos( this.t) - Math.cos( this.alpha)) + M*M*Math.sin(this.t) + Math.sin( this.alpha)))/(M*M+1);
		
		Point2D.Double p = new Point2D.Double( x, y);
		
		return p;
	}


	/**
	 * x座標を与えてシムソン線上の y座標を得ます。
	 * @param x
	 * @return
	 */
	public double getPointOnSimsonLineFromX( double x){
		Point2D.Double pr = this.getR();
		Point2D.Double pq = this.getQ();
		
		double a = (pr.y - pq.y)/(pr.x - pq.x);
		double b = pr.y - a * pr.x;
		
		return a*x+b;
	}

	/**
	 * y座標を与えてシムソン線上の x座標を得ます。
	 * @param y
	 * @return
	 */
	public double getPointOnSimsonLineFromY( double y){
		Point2D.Double pr = this.getR();
		Point2D.Double pq = this.getQ();
		
		double a = (pr.y - pq.y)/(pr.x - pq.x);
		double b = pr.y - a * pr.x;
		
		return y-b/a;
	}


	/*
	 * L の値を得ます。
	 */	
	private double getL(){
		return ((Math.sin( this.alpha) - Math.sin( this.beta))/( Math.cos( this.alpha) - Math.cos( this.beta)));
	}
	
	/*
	 * M の値を得ます。
	 */
	private double getM(){
		return ((Math.sin( this.alpha) - Math.sin( this.gamma))/(Math.cos( this.alpha) - Math.cos( this.gamma)));
	}
	
}
