/*
 * 作成日: 2006/12/08
 *
 * この生成されたコメントの挿入されるテンプレートを変更するため
 * ウィンドウ > 設定 > Java > コード生成 > コードとコメント
 */
package jp.maxwell.applet.simson;

import java.awt.*;
import java.awt.event.*;

/**
 * @author koizumi
 *
 * この生成されたコメントの挿入されるテンプレートを変更するため
 * ウィンドウ > 設定 > Java > コード生成 > コードとコメント
 */
public class SimsonLineApplet extends PhysicsApplet implements ActionListener, AdjustmentListener {
	// 背景色
	private final Color bgColor = new Color( 255, 255, 255);

	// キャンバス
	private SimsonCanvas sc;
	
	// コンポーネント
	private Button startButton;
	// 点 B 位置調整用スクロールバー
	private Scrollbar scbarB;
	// 点 C 位置調整用スクロールバー
	private Scrollbar scbarC;
	
	// シムソン線
	private SimsonLine sl;

	// 初期角度(度)
	private int initAlpha = 0;
	private int initBeta = 120;
	private int initGamma = 270;
	
	private boolean isDraw = false;
	
	/*
	 * 初期化処理
	 */
	public void init(){
		// レイアウトの設定
		this.setLayout( new BorderLayout());
		
		// キャンバスの作成とセット
		this.sc = new SimsonCanvas();
		this.sc.setBackground( this.bgColor);
		this.add( this.sc, BorderLayout.CENTER);
		
		// コントロール用パネル
		Panel p = new Panel();
		p.setLayout( new GridLayout( 1, 3));
		
		// スタートボタン
		this.startButton = new Button("draw");
		// 点 B 用スクロールバー (10度刻みで調整)
		this.scbarB = new Scrollbar( Scrollbar.HORIZONTAL, this.initBeta/10, 2, 0, 38);
		// 点 C 用スクロールバー (10度刻みで調整)
		this.scbarC = new Scrollbar( Scrollbar.HORIZONTAL, this.initGamma/10, 2, 0, 38);
		
		// パネルに追加
		p.add( this.startButton);
		p.add( this.scbarB);
		p.add( this.scbarC);
		
		// パネルをアプレットコンテナに追加
		this.add( p, BorderLayout.NORTH);
		
		// リスナの登録
		this.startButton.addActionListener( this);
		this.scbarB.addAdjustmentListener( this);
		this.scbarC.addAdjustmentListener( this);
		
		// シムソン線の作成
		this.sl = new SimsonLine( this.initAlpha, this.initBeta*Math.PI/180.0, this.initGamma*Math.PI/180.0, 0);
		
		// 描画を設定
		this.isDraw = true;
	}

	/* (非 Javadoc)
	 * @see jp.maxwell.applet.simson.PhysicsApplet#routine()
	 */
	public void routine() {
		if( t > Math.PI*2.0){
			this.isDraw = false;
		}
		if( isDraw){
			sl.setT( 2*Math.PI/360.0*dt*t);
			// シムソン線を描画
			this.sc.drawSimsonLine( sl);
			// 三角形を描画
			this.sc.drawTriangle( this.sl);
			// 九点円を描画
			this.sc.drawNinePointCircle( this.sl);
		}
		else{
			try{
				Thread.sleep( 1000);
			}
			catch( Exception ex){
				// Nothing to do.
			}
		}
	}

	/*
	 * 基礎描画を行います。
	 *
	 */
	private void drawBase(){
		// 画面をクリア
		this.sc.clear();
		// 基礎部品を描画
		this.sc.drawBase();
		// 三角形を描画
		this.sc.drawTriangle( this.sl);
		// 九点円を描画
		this.sc.drawNinePointCircle( this.sl);
	}
	
	/**
	 * スクロールバーの値が変化したときに呼ばれます。
	 * @param ev
	 */	
	public void adjustmentValueChanged( AdjustmentEvent ev){
		this.isDraw = false;
		this.t = 0;
		double alpha = this.sl.getAlpha();
		double beta =  this.sl.getBeta();
		double gamma = this.sl.getGamma();
		if( ev.getSource() == this.scbarB){
			beta = this.scbarB.getValue()*10*Math.PI/180.0;
		}
		else if( ev.getSource() == this.scbarC){
			gamma = this.scbarC.getValue()*10*Math.PI/180.0;
		}
		this.sl = new SimsonLine( alpha, beta, gamma, this.t);
		
		this.drawBase();
	}
	
	/**
	 * スタートボタンが押されたときに呼ばれます。
	 * @param ev
	 */
	public void actionPerformed( ActionEvent ev){
		if( ev.getSource() == this.startButton){
			// 時間をもとに戻す
			this.t = 0;
			this.sl = new SimsonLine( this.sl.getAlpha(), this.sl.getBeta(), this.sl.getGamma(), this.t);

			this.drawBase();

			this.isDraw = true;
		}
	}
}
